//-------------------------------------------------------------------------------------------
// Project            Palette : ForeignControls.palette
// Class              ForeignControl
// Creator            Christian Rapp
// Maintainer         Christian Rapp
// Creation Date      Fri Oct  20 16:57:50 MET 1995
//
// Copyright (C) Uptime Object Factory Inc - All rights reserved.
//
// $Id: ForeignControl.h,v 1.1.1.1 1996/03/18 12:53:55 cr Exp $
//-------------------------------------------------------------------------------------------
/* Description:
The ForeignControl class is an abstract subclass of Control. It is designed to manage up to 3 Control subclass instances, depending on its subclass. If the ForeignControl subclass is a ForeignTextField then two Control subclasses are managed (ControlledTextField, Button) and in case of a ForeignPopUp the managed Control subclasses are a ControlledTextField and a Button.
Managing means to keep this "Sub-Controls" as instance variables and organize their positioning and drawing. An other task of a ForeignControl is to administer the current value (objectValue). The objectValue is an enterprise object. With the key-value coding feature and the knowledge of the property names it is possible to access all information the ForeignControl instance needs. The database access for this enterprise object is handled here as well. ForeignControl connects according to the "description" (modelname.entityname) to a DataSource which corresponds to the KeySearchableDataSource protocol and fetches with the knowledge of MainProperty and CodeProperty the data.
*/
//-------------------------------------------------------------------------------------------

#import "ForeignControlsGlobal.h"
#import <appkit/appkit.h>
#import <eoaccess/eoaccess.h>
#import "ForeignControlDelegate.h"
#import "KeySearchDataSource.h"
#import "Object+ExtendedKeyValueCoding.h"
#import "NSObject+ExtendedKeyValueCoding.h"
#define ID id

typedef enum {LEFT,RIGHT,ABOVE,BELOW} CodeControlPlacement;


@interface ForeignControl:Control
{
	id				nextText;		// outlet to the next textfield
	ID				previousText;	// outlet to the previous textfield
	id				textDelegate;	// delegate for text delegate messages
	Control			*mainControl;	// main Control (TextField or PopUp Button)
	Control			*codeControl;	// optional code TextField  
	Control			*extControl;	// chooser Button for ForeignTextField only
	CodeControlPlacement	codePlacement;	// codeControl position relative to the mainControl

	NSString		*descriptionString;	// modelname and entityname (dot separated)
	NSString		*mainProperty;		// main property name
	NSString		*codeProperty;		// code property name
	BOOL			associationUsesObjectValues;// flag if the association works with object or code value
	BOOL			isNullAllowed;		// flag if an empty field is allowed 
	id<KeySearchableDataSource>	dataSource;	// internal DataSource
	ID				objectValue;		// the current value (an enterprise object)

	ID				delegate;		// internal delegate (only for associations used)
	
	id				target;		// The object that is sent the ForeignControl's action
	SEL				action;		// The message that the ForeignControl sends to its target

	BOOL			isDirty;	// internal flag if the graphic is up to date
	BOOL			isValueBeingChangingProgrammatically;	// internal flag.
}

//===========================================================================================
//==== Class methods
//===========================================================================================
//-------------------------------------------------------------------------------------------
+ initialize;

//===========================================================================================
//==== Initializing and Deallocating of Instances
//===========================================================================================

- initFrame:(const NXRect *)frameRect;
	// initFrame: is the designated initializer. initFrame: sets the instance variables to the following Defaults:
	// associationUsesObjectValues = YES
	// isNullAllowed = YES
	//
	// initFrame: returns self.

- free;
	// Frees the instance variables which are: mainControl, codeControl, extControl, descriptionString, mainProperty, codeProperty and dataSource;
	//
	// See also:  - release (NSObject protocol), - autorelease (NSObject protocol)


//===========================================================================================
//==== Overridden superclass methods
//===========================================================================================

- setAction:(SEL)aSelector;
	// Sets the action instance variable to aSelector. If the subclass is ForeignTextField, the action is sent to the target in only one case. If the user leaves the Control by pressing Return and not by pressing Tab, Shift-Tab or Cmd-Tab. If the subclass is ForeignPopUp the action is sent every time the user changed the popup or leaves the Code field by pressing Return. Returns self.
	//
	// See also:  - action, - setTarget:, - sendAction:to: (Control)
	
- (SEL)action;
	// Returns the action selector. 
	//
	// See also:  :  - setAction:, - target
	
- setTarget:anObject;
	// Sets the target to anObject. This is the object that is sent the action method. Returns self.
	//
	// See also:   - target, - setAction:
	
- target;
	// Returns the target, the object that is sent the action method.
	//
	// See also:  - setTarget:, - action
	
- setNextText:anObject;
	// Sets up anObject as the object whose text will be selected when the user presses Tab while editing the ForeignTextField's text. anObject should respond to the selectText: message. If anObject also responds to both selectText: and setPreviousText:, it's sent setPreviousText: with the receiving ForeignTextField as the argument. This builds a two-way connection, so that pressing Tab in the ForeignTextField selects anObject's text, and pressing Shift-Tab in anObject selects the ForeignTextField's text. Returns self.
	//
	// See also:  - nextText, - setPreviousText:
	
- nextText;
	// Returns the object whose text is selected when the user presses Tab while editing the ForeignTextField. If that object responds to the selectText: message, the current ForeignTextField is deactivated and selectText: is send to the next text.
	//
	// See also:  - setNextText:
	
- setPreviousText:anObject;
	// Sets up anObject as the object whose text will be selected when the user presses Shift-Tab while editing the ForeignTextField's text. anObject should respond to the selectText: message. Your code should not need to use this method directly, since it's invoked automatically by setNextText:. In contrast to setNextText: this method does not build a two-way connection. Returns self.	
	//
	// See also:  - setNextText:

- selectText:sender;
	// Selects the default sub control

- (BOOL)acceptsFirstResponder;
	// Returns YES if the ForeignControl is enabled and accepts the role of first responder for its Window.
	
- forward:(SEL)aSelector :(marg_list)argFrame;
	// In order to offer the same functionality as TextField the ForeignControl has to forward the text delegate messages to its textDelegate. This method receives all method calls which the ForeignControl subclass does not understand and forwards them to the textDelegate, if they have got a "text" prefix and the textDelegate responds to them. You never invoke this method directly.
	//
	// See also:  - respondsTo:
	
- (BOOL) respondsTo:(SEL)aSelector;
	// The method respondsTo: has some extended functionality been added. respondsTo: returns YES if the ForeignControl subclass implements the method (aSelector) or the textDelegate does implement it.

- textDidChange:sender;
	// textDidChange: sets the flag isDirty to YES to make sure that the ForeignControl is updated. You never invoke this method directly. It is invoked by one of the subcontrols only. Returns self

//===========================================================================================
//==== Accessor methods
//===========================================================================================

- (void)setDescriptionString:(NSString *)description;
	// Sets the description of model and entity. The description is expected in the form: @"modelname.entityname". If the flag associationUsesObjectValues is set to YES then the model has to be the same as the connected EOController is currently using and the entity has to be one which has a relationship (connected in EOModeler) to EOController's entity. If the flag associationUsesObjectValues is set to NO then a different model and entity can be chosen without any connections.
	//
	// See also:  - descriptionString, - setAssociationUsesObjectValues:

- (NSString *)descriptionString;
	// Returns the description of the model and the entity as NSString in the form @"modelname.entityname".
	//
	// See also:  - setDescriptionString:

- (id<KeySearchableDataSource>)dataSource;
	// Returns the current used DataSource instance.

- (void)setMainProperty:(NSString *)aPropertyString;
	// Sets the mainProperty. This property has to be in the model and entity which is described in the descriptionString that defines which property should be displayed in the main control. (in case of ForeignTextField the main control is a textfield and in case of ForeignPopUp it is a popup)
	//
	// See also:  - mainProperty

- (NSString *)mainProperty;
	// Returns the main property name.
	//
	// See also:  - setMainProperty:
	
- (void)setCodeProperty:(NSString *)aPropertyString;
	// Sets the codeProperty. This property has to be in the entity which is described in the descriptionString that defines which property should be displayed in the code control. If the flag associationUsesObjectValues is set to NO, the relationship is made through this property. In this case the ForeignControl subclass uses an ForeignCodeAssociation instance. This builds a virtual relationship to the entity which the connected EOController is currently using.
	//
	// See also:  - codeProperty, - setAssociationUsesObjectValues:

	
- (NSString *)codeProperty;
	// Returns the code property name.
	//
	// See also:  - setCodeProperty:

- (void)setCodePlacement:(CodeControlPlacement)aPlacement;
	// Sets the codeControl position relative to the mainControl (LEFT,RIGHT,ABOVE,BELOW)
	//
	// See also:  - codePlacement
	
- (CodeControlPlacement)codePlacement;
	// Returns the codeControl position relative to the mainControl (LEFT,RIGHT,ABOVE,BELOW)
	//
	// See also:  - setCodePlacement:

- (Control *)mainControl;
	// Returns a pointer to the mainControl. You never invoke this method directly. It is only for internal use.
	
- (Control *)codeControl;
	// Returns a pointer to the codeControl. You never invoke this method directly. It is only for internal use.

- (void)setExtControl:(Control *)aControl;
	// Sets the extControl (this is only used in the ForeignTextField subclass as chooser button ). You never invoke this method directly. It is only for internal use.
	//
	// See also:  - extControl

- (Control *)extControl;
	// Returns a pointer to the extControl. You never invoke this method directly. It is only for internal use.
	//
	// See also:  - setExtControl:

- (void)setAssociationUsesObjectValues:(BOOL)yn;
	// The ForeignControl can work together with the EOController in two different modes.
	// If associationUsesObjectValues is YES, then the ForeignControl subclass uses ForeignObjectAssociations. This means that the connection you form from the EOController to the ForeignControl has to be a relationship you have previously declared in the EOModel. In this mode the association works with enterprise objects as value. This has one important benefit that allows you to use another attribute to form your relationship than the one displayed in the code field. As a result you can use the code for something more descriptive. e.g. You have got a two tables a customer table with an attribute "countryId" and an country table with attributes "countryId","countryCode","countryName" (e.g. {"11","CH","Switzerland"}, {"12","D","Germany"} ). In your EOModel you made an one to one relation from customer.countryId to country.countryId. Now you can use in your customer inspector a ForeignControl subclass and connect it with this relationship, define countryName as your main property and countryCode as your code property. In this case the user can work with two different terms which more descriptive instead of a poor number. But always remember the code has to be unique.
	// ATTENTION: If the user changes something in this mode, you have to set yourself a join attribute, which has been used to build up this relationship, to the right value.
	// If associationUsesObjectValues is NO, there is no need for a relationship. In this case an ForeignCodeAssociation instance is used and the ForeignControl subclass forms the relationship between the two tables. Now the association only works with an attribute value. e.g. This means in the example form above, that you connect your EOController to the ForeignControl subclass and choose countryId. In the ForeignControl you declare the countryId as the code property and the countryName as the main property. This might be a way to form a relationship to an other database with a different model.
	//
	// See also:  - associationUsesObjectValues

- (BOOL)associationUsesObjectValues;
	// Returns the state if the ForeignControl subclass works with object values or attribute value.
	//
	// See also:  - setAssociationUsesObjectValues:
	
- (void)setIsNullAllowed:(BOOL)aState;
	// Sets the isNullAllowed flag which allows the user to leave the ForeignControl subclass with an empty value.
	//
	// See also:  - isNullAllowed
	
- (BOOL)isNullAllowed;
	// Returns YES if ForeignControl subclass can be left with an empty field.
	//
	// See also:  - setIsNullAllowed:

- (void)setObjectValue:anObject;
	// Sets a new ForeignControl value (an enterprise object). (This method is used by the ForeigenObjectAssociation to set the value.)
	//
	// See also:  - objectValue, - setCodeValue:, - codeValue

- objectValue;
	// Returns the objectValue of the ForeignControl. This value is also valid if the ForeignControl value has been set using the setCodeValue: method.
	//
	// See also:  - setObjectValue:, - setCodeValue:, - codeValue
	
- (void)setCodeValue:(NSString *)aCodeString;
	// Sets a new ForeignControl value (a code string). This method is used by the ForeigenCodeAssociation to set the values. First it looks for de corresponding enterprise object and sets then the value. setCodeValue: does not use the setObjectValue: method to set the value.
	//
	// See also:  - objectValue, - setObjectValue:, - codeValue

- (NSString *)codeValue;
	// Returns the code string.
	//
	// See also:  - objectValue, - setObjectValue:, - setCodeValue:
	
- updateFromDatabase:sender;
	// Forces a recache in the DataSource.

//===========================================================================================
//==== EOF Connection
//===========================================================================================

- (Class)associationClass;
	// Returns a string with the class name of the association which should be used. This depends on the associationUsesObjectValues flag. If associationUsesObjectValues is YES, @"ForeignObjectAssociation" is returned else @"ForeignCodeAssociation".

//===========================================================================================
//==== Internal methods
//===========================================================================================

- (BOOL)textWillChange:textObject;
	// You never invoke this method directly. It's only for internal use.

- (void)setDelegate:(id <ForeignControlDelegate>)anObject;
	// Sets the internal delegate. This is used to work together with the association, because in some situations occur changes in which no action is send. In this case the delegate which has to conform to the ForeignControlDelegate protocol, receives a controlDidChange: message.
	//
	// See also:  - delegate

- (id <ForeignControlDelegate>)delegate;
	// Returns the internal delegate
	//
	// See also:  - setDelegate:

- (void)_updateFrame;
	// Sets the sub controls to a new position. This method is only in two different cases invoked. After a resize session in IB or after a change of the code placement. You never invoke this method directly. It is only for internal use.
	//
	// See also:  - setCodePlacement:
	
- (TextField *)_createCodeField:(NXRect *)aFrame;
	// Creates and initializes a new code field. Is used while working in IB and in the initFrame: method. You never invoke this method directly. It is only for internal use.
	//
	// See also:  - initFrame:

- _lookUpObjectWithCode:(NSString*)aCode;
	// Returns the enterprise object with the same code property value as aCode. You never invoke this method directly. It is only for internal use.

- _lookUpObjectWithString:(NSString*)aString;
	// Returns the enterprise object with the same main property value as aString. You never invoke this method directly. It is only for internal use.

- (void)_setObjectValue:anObject;
	// Sets a new ForeignControl value (an enterprise object), but does not update the graphic. This method is invoked from setObjectValue:. You never invoke this method directly. It is only for internal use.
	
- (NSString *)_mainValue;
	// Returns the main value as NSString. You never invoke this method directly. It is only for internal use.
	
- _editingControl;
	// If an editing session is active, _editingControl returns an id of the sub control which is currently being editing. You never invoke this method directly. It is only for internal use.

- _selectControls:sender;
	// Selects the ForeignControl. This method is here in this class not implemented and has to be overridden in the subclass.

- _deselectControls:sender;
	// Deselects the ForeignControl. _deselectControls: is only invoked if an new value is set programmatically while an editing session is active.
	
- _updateControls:sender;
	// Updates the graphics of the ForeignControl.

//===========================================================================================
//==== Archiving
//===========================================================================================

- awake;
	// Is invoked after the object has finished unarchiving and only sets the isDirty to YES to make sure that the ForeignControl will be updated.
	//
	// See also:  - read:, - write:
	
- read:(NXTypedStream *)stream;
	// Reads an archived object form a typed stream.
	//
	// See also:  - write:, - awake

- write:(NXTypedStream *)stream;
	// Writes the object to a typed stream.
	//
	// See also:  - read:, - awake


@end