//-------------------------------------------------------------------------------------------
// Project            ForeignControls.palette
// Class              ForeignTextField
// Creator            Christian Rapp
// Maintainer         Christian Rapp
// Creation Date      Fri Oct 27 15:36:39 MET 1995
//
// Copyright (C) Uptime Object Factory Inc - All rights reserved.
//
// $Id: ForeignTextField.h,v 1.1.1.1 1996/03/18 12:53:56 cr Exp $
//-------------------------------------------------------------------------------------------
/* Description:
ForeignTextField is a subclass of the abstract class ForeignControl. ForeignTextField contains two text fields, the main field and the code field and a button on the right side next to the main field. The code field is optional and can be omitted.
ForeignTextField is especially suitable for large amount of datasets (>20) using EOF. The following example illustrates the functionality of the ForeignTextField. We assume we have a database table customer which consits of the attribute last name among others. The ForeignTextField is configured with the last name attribute. Therefore the user can enter any name and the ForeignTextField tests 
1. ...whether the name exists
2. ...the completeness of the name.
3. ...if the entered name is unique.
If all tests succeed the ForeignTextField acts like a normal TextField. In case a test fails a chooser panel appears displaying all possible expansions for your input. If there are any expansions found you can choose one with the mouse or with the cursor arrows otherwise you may delete some characters until one or more expansion is possible.
With the code field you have the possibility to enter a unique e.g. customer id or a department id directly. 
The ForeignTextField offers all usual features of a TextField like textDelegate, nextText and previousText. (Tab and Shift-Tab is supported as usual) To select the field, you can use the mouse or press Cmd-Tab. In IB you can choose which field (main or code) should be default, when it is selected through next text. This setting can be changed (overridden) later with a dwrite. For each  ForeignTextField with the same description ("modelname.entityname") there can be a dwrite. 

    MyApp "codefield is defaultfield PEOPLE.DEPARTMENT" YES

The chooser panel uses dwrites too. They also depend on model and entity.

    MyApp "NXWindow Frame Chooser PEOPLE.DEPARTMENT" "891 733 357 255 0 0 1532 1200 "

ForeignTextField can be completely used without the mouse.

IMPORTANT: ForeignTextField needs some resources, which you have to add to your project. Two images (LOV.tiff and LOVH.tiff), a NIB-file (ForeignChooser.nib) and the palette library (libForeignControls.a). Those resources were delivered together with the palette.
*/
//-------------------------------------------------------------------------------------------

#import "ForeignControlsGlobal.h"
#import <appkit/appkit.h>
#import <eointerface/eointerface.h>
#import "ForeignControl.h"

#define GET_TEXTFIELD_RECT(a)		(NXSetRect((a), 0, 0, 99, 21))
#define GET_CODE_RECT(a)		(NXSetRect((a), 0, 0, 40, 21))
#define GET_CHOOSERBUTTON_RECT(a)	(NXSetRect((a), 0, 0, 21, 21))

@interface ForeignTextField : ForeignControl
{
	NSArray			*chooserKeys;	// List of the in the, chooser displayed, properties
	NSString		*chooserTitle;	// Choosers title
	BOOL			isCodeDefault;	// Flag if the code field is the default field
	BOOL			isCodeEditing;	// Flag	which field currently is edited
	BOOL			endEditingIsAllowedBecauseOfInternalChange; // Flag if the untested leaving of textWillEnd: is allowed

	NXEvent			endEditingEvent;
}

//===========================================================================================
//==== Initializing and Deallocating of Instances
//===========================================================================================

+ initialize;
	// ForeignTextField needs two images to draw itself. This images are loaded here and are as shared instances available for every ForeignTextField instance. At this point the class allocates also a global shared panel which receives Cmd key events and notifies the ForeignTextField instance if a Cmd-Tab has occurred.

- initFrame:(const NXRect*)aRect;
	// Returns self. Is the designated initializer.
	
- free;
	// Frees the instance

//===========================================================================================
//==== Overridden superclass methods
//===========================================================================================

- (id<KeySearchableDataSource>)dataSource;
	// Returns the current used DataSource instance. When this method is invoked for the first time it also checks the defaults database and sets the default field (main field or code field).
	// Such a dwrite can look like that:
	//
	//    MyApp "codefield is defaultfield PEOPLE.DEPARTMENT" YES

- hitTest:(NXPoint *)aPoint;
	// Determines whether aPoint lies whithin a sub control. If there is a sub control within aPoint lies the sub control is returned otherwise nil.  
	// If the main or the code field is hit by the point the correct background colors for the fields are set. This method is only invoked from the window.

- resizeSubviews:(const NXSize *)oldSize;
	// This method resizes the ForeignTextField and moves the button to its correct place next to the main field. Returns self.
	
- setEnabled:(BOOL)flag;
	// Enables or disables the ForeignTextField's ability to receive mouse and keyboard events, according to the value of flag. Returns self.

//===========================================================================================
//==== Accessor methods
//===========================================================================================

- (NSArray *)chooserKeys;
	// Returns an array with the names of attributes which should be displayed in the chooser.
	//
	// See also:  - setChooserKeys:
	
	
- (void)setChooserKeys:(NSArray *)theKeys;
	// Sets the names of attributes which should be displayed in the chooser.
	//
	// See also:  - chooserKeys
	
- (NSString*)chooserTitle;
	// Returns the title string of the chooser.
	//
	// See also:  - setChooserTitle:
	
- (void)setChooserTitle:(NSString*)aTitle;
	// Sets the title string of the chooser.
	//
	// See also:  - chooserTitle
	
- (BOOL)isCodeDefault;
	// Returns YES if the code field is the default field.
	//
	// See also:  - setIsCodeDefault:
	
- (void)setIsCodeDefault:(BOOL)aState;
	// Sets which field should be the default field.
	//
	// See also:  - isCodeDefault

//===========================================================================================
//==== Delegate methods
//===========================================================================================

- (BOOL)textWillEnd:textObject;
	// You never invoke this method directly. It is only for internal use.
	
- textDidEnd:sender endChar:(unsigned short)whyEnd;
	// You never invoke this method directly. It is only for internal use.
	
- (BOOL)textWillGetEscapeKey:textObject;
	// You never invoke this method directly. It is only for internal use.
	

//===========================================================================================
//==== Internal methods
//===========================================================================================

- _delayedEndEditing:sender;
	// You never invoke this method directly. It's only for internal use.

- _runChooser:sender;
	// Invokes _runChooserWithString with the current value of the main field. You never invoke this method directly. It is only for internal use. Returns self.
	
- _runChooserWithString:(NSString *)newValue;
	// Runs the ForeignChooser modal session. You never invoke this method directly. It is only for internal use. Returns self.

- _selectControls:sender;
	// Makes the correct field to the firstResponder and selects it. You never invoke this method directly. It is only for internal use. Returns self.

- _updateControls:sender;
	//	If the isDirty is YES, then _updateControls: updates the values of the subcontrols. You never invoke this method directly. It is only for internal use. Returns self.
	
- _updateBackground:sender;
	// Sets the backgound to the right color. You never invoke this method directly. It is only for internal use. Returns self.

- (BOOL)_mainTextWillEnd:textObject;
	// This method is invoked from the textWillEnd: delegate method, if the main field has been edited and verifies the input. Returns NO if the user can leave the field otherwise YES. You never invoke this method directly. It is only for internal use.
	
- (BOOL)_codeTextWillEnd:textObject;
	// This method is invoked from the textWillEnd: delegate method, if the code field has been edited and verifies the input. Returns NO if the user can leave the field otherwise YES. You never invoke this method directly. It is only for internal use.
	
- (BOOL)textWillGetCmdSequence:sender withKey:(unsigned short)charCode;
	// This method is invoked when the user has pressed Cmd-Tab and forces a change of the current editing field. You never invoke this method directly. It's only for internal use. Returns self.

//===========================================================================================
//==== Archiving
//===========================================================================================

- awake;
	// Reads the defaults database to determine which field is the default field. Returns self.
	//
	// See also:  - read:, -write:
	
- read:(NXTypedStream *)stream;
	// Reads an archived object form a typed stream. Returns self.
	//
	// See also:  - awake:, -write:
	
- write:(NXTypedStream *)stream;
	// Writes the object to a typed stream. Returns self.
	//
	// See also:  - read:, -awake:
	

@end